﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win;
using System.Data;
using Infragistics.Win.UltraWinGrid;
using System.Drawing;
using System.Diagnostics;
using Infragistics.Win.UltraWinEditors;

namespace wfaTestHeaderCombobox
{
    /// <summary>
    /// Class, which implements IUIElementCreationFilter interface
    /// </summary>
    /// <remarks>I used example from http://devcenter.infragistics.com/download/samples/kb/ultrawingrid_checkbox_on_header_cs.zip
    /// for my class creation</remarks>
    internal class ComboboxOnHeaderCreationFilter : IUIElementCreationFilter
    {
        /// <summary>
        /// Data source table
        /// </summary>
        private DataTable _source;
        private UltraGrid control = null;
        private EditorOwner embeddableEditor = null;
        private EditorWithCombo editor = null;
        private UltraCombo combo = null;

        #region IUIElementCreationFilter Members

        void IUIElementCreationFilter.AfterCreateChildElements(UIElement parent)
        {
            if (parent is HeaderUIElement) 
			{                 
				HeaderBase aHeader = ((HeaderUIElement)parent).Header;
                
				if (aHeader.Caption == "Account")
				{
					TextUIElement aTextUIElement;  
                    EditorWithComboUIElement aComboBoxUIElement = (EditorWithComboUIElement)parent.GetDescendant(typeof(EditorWithComboUIElement));                       

                    if ( this.combo == null )
                    {
                        this.combo = new UltraCombo(); 
                        this.combo.DisplayStyle = EmbeddableElementDisplayStyle.WindowsVista;                    
                        this.combo.DataSource = _source;
                        this.combo.DisplayMember = "Name";
                        this.combo.ValueMember = "Code";                    
                        this.combo.SelectedRow = combo.Rows[0];
                    }

                    if ( this.embeddableEditor == null )
                        this.embeddableEditor = new EditorOwner(this.control, this.combo); 
                    
                    if ( this.editor == null )
                        this.editor = new EditorWithCombo( new EditorOwner(this.control, this.combo) );

                    if (aComboBoxUIElement == null)
                        aComboBoxUIElement = new EditorWithComboUIElement(parent, embeddableEditor, editor, null, true, true, true, true);                        
                    else
                    {
                        embeddableEditor = aComboBoxUIElement.Owner as EditorOwner;
                        editor = aComboBoxUIElement.Editor as EditorWithCombo;
                    }                    

					aTextUIElement = (TextUIElement)parent.GetDescendant(typeof(TextUIElement));			
                    if ( aTextUIElement == null )
                        return;

                    ColumnHeader aColumnHeader = 
                    aComboBoxUIElement.GetAncestor(typeof(HeaderUIElement)).GetContext(typeof(Infragistics.Win.UltraWinGrid.ColumnHeader)) as ColumnHeader;

                    aComboBoxUIElement.Editor.ValueChanged += new EventHandler(Editor_ValueChanged);
					parent.ChildElements.Add(aComboBoxUIElement); 
                    aComboBoxUIElement.Rect = new Rectangle(parent.Rect.X + 3, parent.Rect.Y + (parent.Rect.Height - 20) - 3, parent.Rect.Width - 6, 20);
                    aTextUIElement.Rect = new Rectangle(aTextUIElement.Rect.X, aTextUIElement.Rect.Y + 2, aTextUIElement.Rect.Width, 18);

                    // I don't know how automatically widen header's height, when sum of combobox height and header's caption text greater column's 
                    // header height
                    if (aHeader.Height <= aComboBoxUIElement.Rect.Height + aTextUIElement.Rect.Height)
                    {
                        aHeader.Band.ColHeaderLines = aHeader.Band.ColHeaderLines + 1;
                    }
				}
				else
				{
                    EditorWithComboUIElement aCheckBoxUIElement = (EditorWithComboUIElement)parent.GetDescendant(typeof(EditorWithComboUIElement));

					if(aCheckBoxUIElement != null)
					{
						parent.ChildElements.Remove(aCheckBoxUIElement);
						aCheckBoxUIElement.Dispose();
					}
				}                
			}            
        }

        void Editor_ValueChanged(object sender, EventArgs e)
        {
            Debug.WriteLine("Header's combobox value changed");
        }

        bool IUIElementCreationFilter.BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }

        /// <summary>
        /// .ctor
        /// </summary>
        /// <param name="source">Data source table for header's combobox</param>
        internal ComboboxOnHeaderCreationFilter(DataTable source, UltraGrid control)
        { 
            _source = source;
            this.control = control;
        }

        #endregion
    }

    public class EditorOwner : DefaultEditorOwner
    {
        UltraGrid control = null;
        UltraCombo combo = null;

        public EditorOwner( UltraGrid control, UltraCombo combo )
        {
            this.control = control;
            this.combo = combo;
        }

        public override System.Windows.Forms.Control GetControl(object ownerContext)
        {
            return this.control;
        }

        public override IValueList GetValueList(object ownerContext)
        {
            return this.combo;
        }
    }
}
